#!/usr/bin/perl

=head1 NAME

abook.cgi - LDAP addressbook user interface

=head1 DESCRIPTION

abook.cgi provides a web interface to the LDAP addressbook.  It allows the user
to search for, display, create, edit, and delete entries.

=head2 Notes

=over 4

=item *

All updates to LDAP backend are done using the manager dn defined in
the AddressBook configuration file.  For security, it is advisable to use a web server
authentication mechanism so that only trusted users can access the cgi.

=back

=head1 AUTHOR

David Leigh dleigh@sameasiteverwas.net

=head1 SEE ALSO

Abook

=cut

use CGI param,header;
use AddressBook;
use Data::Dumper;
use vars qw($VERSION);
$VERSION = '1.00';

my $config_file='/usr/local/etc/AddressBook.conf';
my $query = CGI::new();
my $myURL=$query->url;
my $op = $query->param(op);
my $entry;
my $error_msg = "";
my ($db,$entry_db,$search_db,$newEntry_db,$config);

printHeader();

$entry = parseEntry($query);
print "<table border width=100%>\n";
print "<tr><td align=center>\n";
if ($op eq "displayEntry") {displayEntry($entry)}
elsif ($op eq "Modify") {editEntry($entry)}
elsif ($op eq "submitModify") {modifyEntry($query,$entry)}
elsif ($op eq "searchFor") {searchFor($query)}
elsif ($op eq "newEntry") {newEntry($query)}
elsif ($op eq "confirmDelete") {confirmDelete($entry)}
elsif ($op eq "deleteEntry") {deleteEntry($entry)}
elsif ($op eq "addField") {addField($query,$query->param(fieldName),$entry)}
print "</td><td valign=top align=center>\n";
searchBoxForm();
print "</td></tr></table>\n";

#-------------------------------------------------------------------------
sub config {
  unless ($config) {
    eval {$config = AddressBook::Config->new(config_file=>$config_file)};
    errorExit($@) if $@;
  }
  return $config;
}
#-------------------------------------------------------------------------
sub db {
  unless ($db) {
    eval {$db = AddressBook->new(source => LDAP,config=>config())};
    errorExit($@) if $@;
  }
  return $db;
}
#-------------------------------------------------------------------------
sub entry_db {
  unless ($entry_db) {
    eval {$entry_db = AddressBook->new(source => abookentry,config=>config())};
    errorExit($@) if $@;
  }
  return $entry_db;
}
#-------------------------------------------------------------------------
sub search_db {
  unless ($search_db) {
    eval {$search_db = AddressBook->new(source => abooksearch,config=>config())};
    errorExit($@) if $@;
  }
  return $search_db;
}
#-------------------------------------------------------------------------
sub newEntry_db {
  unless ($newEntry_db) {
    eval {$newEntry_db = AddressBook->new(source => abooknew,config=>config())};
    errorExit($@) if $@;
  }
  return $newEntry_db;
}
#-------------------------------------------------------------------------
sub printHeader {
  print $query->header(-charset=>'UTF-8');
  $jscript = << 'END_SCRIPT';
  var executable
  function setop(formName,val) {
    executable = "document."+formName+".op.value = val"
    eval(executable)
    executable = "document."+formName+".submit()"
    eval(executable)
  }
END_SCRIPT
print $query->start_html(-title=>'Address Book',
			 -script=>{-language=>'Javascript',
				   -code=>$jscript,
				 },
			 );
}
#-------------------------------------------------------------------------
sub parseEntry {
  my $query=shift;
  if ($query->param(entry)) {
    my $attr = eval $query->param(entry);
    my $config = config();
    $entry = AddressBook::Entry->new(config=>$config,attr=>$attr);
    return $entry;
  } else {
    return undef;
  }
}
#-------------------------------------------------------------------------
sub addField {
  my ($query,$fieldName,$entry) = @_;
  my $entry_db = entry_db() || return;
  my $new_entry = $entry_db->read_from_args($query);
  $new_entry->add(db=>$entry_db->{db_name},attr=>{$fieldName=>""});
  editEntry($new_entry,$entry);
}
#-------------------------------------------------------------------------
sub modifyEntry {
  my ($query,$entry) = @_;
  my $entry_db = entry_db() || return;
  my $new_entry = $entry_db->read_from_args($query);
  $new_entry->calculate;
  $entry->chop();
  unless (AddressBook::Entry::compare($entry,$new_entry)) {
    my $db = db() || return;
    if ($db->update(filter=>$entry->get(values_only=>1),entry=>$new_entry)) {
      $lastResult = "Entry modified successfully\n";
    } else {
      $lastResult = "An error has occurred\n";
      errorExit($db->code);
      return;
    }
  } else {
    $lastResult = "No changes were made\n";
  }
  displayEntry($new_entry);
}
#-------------------------------------------------------------------------
sub newEntry {
  my $query=shift;
  my $newEntry_db = newEntry_db() || return;
  my $entry = $newEntry_db->read_from_args($query);
  my $db=db() || return;
  if ($db->add($entry)) {
    editEntry($entry);
    $lastResult = "New entry has been added\n";
  } else {
    errorExit($db->code);
    $lastResult = "An error has occurred\n";
  }
}
#-------------------------------------------------------------------------
sub searchFor() {
  my $query=shift;
  my $db = db() || return;
  my $search_db = search_db() || return;
  my $entry = $search_db->read_from_args($query);
  $entry->calculate;
  my $attr=$entry->get(values_only=>1);
  my @filter = map {{$_=>$attr->{$_}}} keys %{$attr};
  my $count;
  my $query_string;
  if (@filter) {
    $count = $db->search(filter=>\@filter,fuzzy=>1);
    $query_string = $search_db->write($entry);
  } else {
    $count = $db->search();
    $query_string = "All Records";
  }
  if($db->code) {
    errorExit($db->code);
    return;
  }
  if ($count == 0) {
    $lastResult = "No entries were found matching your query: $query_string\n";
  } elsif ($count == 1) {
    $entry = $db->read;
    displayEntry($entry,$query_string);
  } else {
    listBoxForm($count,$query_string);
  }
}
#-------------------------------------------------------------------------
sub listBoxForm {
  my ($entry_count,$query_string) = @_;
  print "<form name=listboxform method=POST action=$myURL>\n";
  print "Click on a name from the list below:\n";
  print "<p><select name=entry size=20 onChange=\"listboxform.submit();\">\n";
  my ($i);
  my $db = db() || return;
  while ($entry = $db->read) {
    $attr = $entry->get(db=>LDAP,values_only=>1);
    $canon_attr = $entry->get(values_only=>1);
    for ($i=0;$i<=$#{$attr->{cn}};$i++) {
      if (defined $attr->{sn}->[$i]) {
	$names->{$attr->{cn}->[$i]}->{sortfield} = $attr->{sn}->[$i];
      } else {
	$names->{$attr->{cn}->[$i]}->{sortfield} = $attr->{cn}->[$i];
      }
      $names->{$attr->{cn}->[$i]}->{entry} = Dumper($canon_attr);
      $name_count++;
    }
  }
  foreach $name (sort {$names->{$a}->{sortfield} cmp $names->{$b}->{sortfield}} keys %{$names}) {
    print "<option value=\"".$names->{$name}->{entry}."\">$name\n";
  }
  print "</select>\n";
  print "<input type=hidden name=op value=displayEntry>\n";
  print "</form>\n";
  $lastResult = "$name_count names found in $entry_count entries for query: $query_string\n";
}
#-------------------------------------------------------------------------
sub searchBoxForm {
  my $options;
  print "<table width=100% height=100% border>\n";
  print "<tr><td align=center valign=center>\n";
  print "<p><b>Search:</b>\n";
  print "<form method=POST action=$myURL>\n";
  print "<input type=hidden name=op value=searchFor>\n";
  my $entry = AddressBook::Entry->new(config=>config());
  my $search_db = search_db() || return;
  $entry->fill(db=>$search_db->{db_name});
  print $search_db->entry_form($entry);
  print "<input type=submit value=\"Search\">";
  print "<p><i>An empty search returns all records</i>\n";
  print "</form>\n";
  print "</td></tr><tr><td align=center valign=center>\n";
  print "<form method=POST action=$myURL>\n";
  print "<p>Create a new entry:\n";
  $entry = AddressBook::Entry->new(config=>config());
  my $newEntry_db = newEntry_db() || return;
  $entry->fill(db=>$newEntry_db->{db_name});
  print $newEntry_db->entry_form($entry);
  print "<input type=submit value=\"Create Entry\">\n";
  print "<input type=hidden name=op value=newEntry>\n";
  print "</form>\n";
  print "</td></tr>\n";
  if ($lastResult = lastResult()) {
    print "<tr><td valign=bottom align=center>\n";
    print "<p><b>Results:</b>\n";
    print "<p><font color=blue>$lastResult</font>\n";
    print "</td></tr>\n";
  }
  print "</table>\n";
}
#-------------------------------------------------------------------------
sub displayEntry {
  my ($entry,$query_string)=@_;
  my $attr = $entry->get(values_only=>1);
  $cn=get_name($entry);
  print "<table align=center>\n";
  print "<tr><td align=center>\n";
  print "<p>Record for <b>$cn</b>:\n";
  print "<hr>\n";
  displayEntryBox($entry);
  print "<hr>\n";
  print "</td></tr></table>\n";
  print "<form method=POST action=$myURL>\n";
  print "<input type=hidden name=entry value=\"" . Dumper($attr) . "\">\n";
  print "<p><input type=submit value=Modify name=op>\n";
  print "</form>\n";
  if ($query_string) {
    $lastResult = "One entry found for your query: $query_string\n";
  }
}
#-------------------------------------------------------------------------
sub get_name {
  my $entry = shift;
  my $attr = $entry->get(db=>LDAP);
  my $cn = join "/", @{$attr->{cn}->{value}};
  return $cn;
}
#-------------------------------------------------------------------------
sub editEntry() {
  my ($entry,$old_entry)=@_;
  my $entry_db = entry_db() || return;
  my $cn = get_name($entry);
  if (! $old_entry) {$old_entry=$entry};
  my $attr = $old_entry->get(values_only=>1);
  $entry->fill(db=>$entry_db->{db_name},defaults=>1);
  $entry->calculate;
  print "<table><tr><td colspan=4>\n";
  print "<p align=center>Record for <b>$cn</b>:\n";
  print "<form method=POST action=$myURL name=\"editform\">\n";
  print $entry_db->entry_form($entry,"editform");
  print "<hr>\n";
  print "</td></tr><tr><td valign=top>\n";
  print "<input type=hidden name=entry value=\"" . Dumper($attr) . "\">\n";
  print "<input type=hidden value=submit name=op>\n";
  print "<input type=button value=\"Submit Changes\" onClick=\"setop('editform','submitModify')\";\">\n";
  print "</td><td valign=top>\n";
  print "<input type=button value=\"Delete Entry\" onClick=\"setop('editform','confirmDelete')\";\">\n";
  print "</td><td valign=top>\n";
  print "<select name=fieldName size=1 onChange=\"setop('editform','addField')\";\">\n";
  print "<option>-- add a form field --\n";
  my $meta;
  foreach ($entry_db->get_attribute_names) {
    $canon_attr_name = $entry_db->{config}->{db2generic}->{$entry_db->{db_name}}->{$_};
    $meta = $entry_db->{config}->getMeta(attr=>$canon_attr_name,db=>$entry_db->{db_name});
    unless ($meta->{non_multiple} =~ /yes/i
	    || $meta->{read_only} =~ /yes/i
	    || $meta->{type} eq "select") {
      print "<option>$_\n";
    }
  }
  print "</select></td>\n";
  print "<td valign=top><input type=reset></td>\n";
  print "</form>\n";
  print "</tr></table>\n";
}
#-------------------------------------------------------------------------
sub deleteEntry {
  my $entry = shift;
  my $cn = get_name($entry);
  my $db = db() || return;
  if ($db->delete($entry)) {
    $lastResult = "Entry for <b>$cn</b> has been deleted.\n";
  } else {
    $lastResult = "An error has occurred\n";
    errorExit($db->code);
  }
}
#-------------------------------------------------------------------------
sub displayEntryBox {
  my $entry=shift;
  my $entry_db = entry_db() || return;
  $entry->fill(db=>$entry_db->{db_name});
  print $entry_db->write($entry);
}
#-------------------------------------------------------------------------
sub confirmDelete {
  my $entry = shift;
  my $cn = get_name($entry);
  my $attr = $entry->get(values_only=>1);
  print "<table align=center>\n";
  print "<tr><td colspan=2 align=center>\n";
  print "Are you sure you want to delete the following entry for <b>$cn</b>:\n";
  print "<hr>\n";
  displayEntryBox($entry);
  print "<hr>\n";
  print "</td></tr>\n";
  print "<tr>\n";
  print "<form method=POST action=$myURL>\n";
  print "<input type=hidden name=entry value=\"" . Dumper($attr) . "\">\n";
  print "<input type=hidden value=deleteEntry name=op>\n";
  print "<td align=center><input type=submit value=Yes></td>";
  print "</form>\n";
  print "<form method=POST action=$myURL>\n";
  print "<input type=hidden name=entry value=\"" . Dumper($attr) . "\">\n";
  print "<input type=hidden value=displayEntry name=op>\n";
  print "<td align=center><input type=submit value=No></td>\n";
  print "</form>\n";
  print "</tr></table>\n";
}
#-------------------------------------------------------------------------
sub lastResult {
  if ($lastResult) {
    my $ret = $lastResult;
    $lastResult = undef;
    return $ret;
  } else {
    return undef;
  }
}
#-------------------------------------------------------------------------
sub errorExit {
  my ($error_mesg) = @_;
  if ($error_mesg =~ /ALREADY_EXISTS/) 
     {$error_mesg = "An address book entry already exists for that name"}
  $error_mesg =~ s/>/\&gt\;/g;
  $error_mesg =~ s/</\&lt\;/g;
  print "<p align=center><font color=red>Error:\n";
  print "<p><pre>$error_mesg</pre></font>\n";
}
